@@ -22,14 +22,39 @@ import os
 import re
 import sys
 import logging
+from functools import partial
 from datetime import datetime
 from abc import ABCMeta, abstractmethod
+import json
 
 from _config import AttrDict
 
-__all__ = ['LogParser', 'AccessLogLine', 'AccessLog', 
-           'CommonLogFormat']
+__all__ = ['multikey_getter_gen', 'LogParser', 'JSONParser', 'LogLine',
+           'AccessLog', 'CommonLogFormat']
 
+
+def multikey_getter_gen(parser, keys, is_indices=False, delimiter="\t"):
+    """Generator meta-function to return a function
+    parsing a logline and returning multiple keys (tab-delimited)"""
+    if is_indices:
+        keys = map(int, keys)
+        
+    def multikey_getter(line, parser, keyset):
+        data = parser(line.strip())
+        return delimiter.join((unicode(data[k]) for k in keyset))
+    
+    def multiindex_getter(line, parser, keyset):
+        data = parser(line.strip())
+        return delimiter.join((data.by_index(idx-1, raw=True) for idx in keys))
+
+    if is_indices is True:
+        # Field indices
+        return partial(multiindex_getter, parser=parser, keyset=keys)
+    else:
+        # Field names
+        return partial(multikey_getter, parser=parser, keyset=keys)
+
+    
 class LogParser(object):
     """Base class for all our parsers"""
     __metaclass__ = ABCMeta
@@ -46,13 +71,31 @@ class LogParser(object):
         """Set a format specifier for parser.
         Some parsers can use this to specify
         a format string"""
-
-class AccessLogLine(dict):
+        
+        
+class LogLine(dict):
     """Instrumented dictionary that allows
-    convenient access to a parsed access_log line,
+    convenient access to a parsed log lines,
     using key-based lookup / index-based / raw / parsed"""
     
-    def __init__(self, fieldnames):
+    def __init__(self, fieldnames=None):
+        """Initialize logline. This class can be reused
+        across multiple input lines by using the clear()
+        method after each invocation"""
+        
+        self._fieldnames = None
+        
+        if fieldnames:
+            self.fieldnames = fieldnames
+            
+    @property
+    def fieldnames(self):
+        """Getter method for the field names"""
+        return self._fieldnames
+            
+    @fieldnames.setter
+    def fieldnames(self, fieldnames):
+        """Set the log format field names"""
         self._fieldnames = dict(enumerate(fieldnames))
         
     def by_index(self, i, raw=False):
@@ -71,7 +114,32 @@ class AccessLogLine(dict):
             val = self[key]
         return val
         
+
+class JSONParser(LogParser):
+    """Parser implementation for JSON format logs"""
+    
+    def __init__(self):
+        LogParser.__init__(self)
+        self._logline_wrapper = LogLine()
         
+    def parse(self, line):
+        """Parse JSON line"""
+        parsed_row = json.loads(line)
+        
+        data = self._logline_wrapper
+
+        # This is called for every log line - This is because
+        # JSON logs are generally schema-less and so fields
+        # can change between lines.
+        self._logline_wrapper.fieldnames = parsed_row.keys()
+            
+        data.clear()
+        for k, v in parsed_row.iteritems():
+            data[k] = v
+
+        return data
+    
+
 class AccessLog(LogParser):
     """Apache access_log logfile parser. This can
     consume arbitrary Apache log field directives. see
@@ -86,12 +154,12 @@ class AccessLog(LogParser):
         
         if format:
             self.fieldselector = self._parse_log_format(format)
-            self._logline_wrapper = AccessLogLine(self.fieldnames)     
+            self._logline_wrapper = LogLine(self.fieldnames)     
 
     def set_format(self, format):
         """Set the access_log format"""
         self.fieldselector = self._parse_log_format(format)
-        self._logline_wrapper = AccessLogLine(self.fieldnames)     
+        self._logline_wrapper = LogLine(self.fieldnames)     
         
     def parse(self, logline):
         """
