@@ -18,11 +18,12 @@ import sys
 import unittest
 import logging
 from tempfile import mkstemp
+from datetime import datetime
 from StringIO import StringIO
 from operator import itemgetter
 
 from logtools import (filterbots, geoip, logsample, logsample_weighted, 
-                      logparse, logmerge, logplot)
+                      logparse, logmerge, logplot, qps)
 from logtools.parsers import *
 from logtools import logtools_config, interpolate_config, AttrDict
 
@@ -40,9 +41,17 @@ class ParsingTestCase(unittest.TestCase):
             '127.0.0.1 - frank [10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0" 200 2326',
             '127.0.0.2 - jay [10/Oct/2000:13:56:12 -0700] "GET /apache_pb.gif HTTP/1.0" 200 2326'
             ]
+        self.json_rows = [
+            '{"key1":"val1","key2":true,"key3":31337,"key4":null,"nested_key":[{"nested_key_1":"2"}]}'
+        ]
+        
+    def testJSONParser(self):
+        parser = JSONParser()
+        for logrow in self.json_rows:
+            parsed = parser(logrow)
+            self.assertNotEquals(parsed, None, "Could not parse line: %s" % str(logrow))
         
     def testAccessLog(self):
-        """Test Apache access_log format parser"""
         parser = AccessLog()
         parser.set_format(format='%h %l %u %t "%r" %>s %b')
         self.assertRaises(ValueError, parser, 'example for invalid format')
@@ -51,20 +60,25 @@ class ParsingTestCase(unittest.TestCase):
             self.assertNotEquals(parsed, None, "Could not parse line: %s" % str(logrow))
             
     def testCommonLogFormat(self):
-        """Test CLF Parser"""
         parser = CommonLogFormat()
         self.assertRaises(ValueError, parser, 'example for invalid format')
         for logrow in self.clf_rows:
             parsed = parser(logrow)
             self.assertNotEquals(parsed, None, "Could not parse line: %s" % str(logrow))        
         
-        
     def testLogParse(self):
         options = AttrDict({'parser': 'CommonLogFormat', 'field': 4})
         fh = StringIO('\n'.join(self.clf_rows))
         output = [l for l in logparse(options, None, fh)]
         self.assertEquals(len(output), len(self.clf_rows), "Output size was not equal to input size!")
         
+    def testMultiKeyGetter(self):
+        parser = parser = CommonLogFormat()
+        func = multikey_getter_gen(parser, keys=(1,2), is_indices=True)
+        fh = StringIO('\n'.join(self.clf_rows))
+        output = [func(l) for l in fh]
+        self.assertEquals(len(output), len(self.clf_rows), "Output size was not equal to input size!")   
+        
             
 class FilterBotsTestCase(unittest.TestCase):
     def setUp(self):
@@ -74,19 +88,29 @@ class FilterBotsTestCase(unittest.TestCase):
             "ip_ua_re": "^(?P<ip>.*?) - USER_AGENT:'(?P<ua>.*?)'",
             "bots_ips": StringIO(),
             "bots_ua": StringIO("\n".join([
+                "## Example comment ##",
                 "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)",
-                "r'.*MSIECrawler'"
+                "ssearch_bot/Nutch-1.0 (sSearch Crawler; http://www.semantissimo.de)",
+                "r'.*crawler'",
+                "s'MSIECrawler)'",
+                "p'DotSpotsBot'",
+                "p'Java/'"
                 ]) + "\n")
         })
         self.fh = StringIO(
             "127.0.0.1 - USER_AGENT:'Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)' - ...\n" \
             "255.255.255.255 - USER_AGENT:'Mozilla' - ...\n" \
-            "1.1.1.1 - USER_AGENT:'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSIECrawler)'\n"
+            "1.1.1.1 - USER_AGENT:'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSIECrawler)'\n" \
+            "2.2.2.2 - USER_AGENT:'Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; .NET CLR 1.1.4322; MSIECrawler)'\n" \
+            "3.3.3.3 - USER_AGENT:'DotSpotsBot/0.2 (crawler; support at dotspots.com)'\n" \
+            "4.4.4.4 - USER_AGENT:'inagist.com url crawler'\n" \
+            "5.5.5.5 - USER_AGENT:'Java/1.6.0_18'\n" \
+            "6.6.6.6 - USER_AGENT:'ssearch_bot/Nutch-1.0 (sSearch Crawler; http://www.semantissimo.de)'\n"
         )
 
     def testFiltering(self):
         i=0
-        for l in filterbots(self.options, None, self.fh): 
+        for l in filterbots(fh=self.fh, **self.options): 
             i+=1
         self.assertEquals(i, 1, "filterbots output size different than expected: %s" % str(i))
 
@@ -107,7 +131,7 @@ class GeoIPTestCase(unittest.TestCase):
             print >> sys.stderr, "GeoIP Python package not available - skipping geoip unittest."
             return
 
-        output = [(geocode, ip, line) for geocode, ip, line in geoip(self.options, None, self.fh)]
+        output = [(geocode, ip, line) for geocode, ip, line in geoip(fh=self.fh, **self.options)]
         self.assertEquals(len(output), 2, "Output size was different than expected: %s" % str(len(output)))
 
         
@@ -126,12 +150,12 @@ class SamplingTestCase(unittest.TestCase):
         ]))
 
     def testUniformSampling(self):
-        output = [r for r in logsample(self.options, None, self.fh)]
+        output = [r for r in logsample(fh=self.fh, **self.options)]
         self.assertEquals(len(output), self.options.num_samples, 
                           "logsample output size different than expected: %s" % len(output))
         
     def testWeightedSampling(self):
-        output = [(k, r) for k, r in logsample_weighted(self.weighted_opts, None, self.fh)]
+        output = [(k, r) for k, r in logsample_weighted(fh=self.fh, **self.weighted_opts)]
         self.assertEquals(len(output), self.weighted_opts.num_samples, 
                           "logsample output size different than expected: %s" % len(output))        
 
@@ -159,6 +183,22 @@ class MergeTestCase(unittest.TestCase):
         self.assertEquals(map(itemgetter(0), output), sorted(map(lambda x: int(x[0]), output)), 
                           "Output was not numerically sorted!")
         
+    def testDateMerge(self):
+        os.write(self.tempfiles[0][0], "\n".join(['2010/01/12 07:00:00,one', '2010/01/12 08:00:00,five', 
+                                                  '2010/01/13 10:00:00,threehundred']))
+        os.write(self.tempfiles[1][0], "\n".join(['2010/01/12 07:30:00,one', '2010/01/12 08:10:00,five', 
+                                                  '2010/01/12 21:00:00,threehundred']))
+        os.write(self.tempfiles[2][0], "\n".join(['2010/01/11 05:33:03,one', '2010/01/12 03:10:00,five', 
+                                                  '2010/01/21 22:00:00,threehundred']))
+        
+        dateformat = '%Y/%m/%d %H:%M:%S'
+        options = AttrDict({'delimiter': ',', 'field': 1, 'datetime': True, 'dateformat': dateformat })
+        output = [(k, l) for k, l in logmerge(options, self.args)]
+        
+        self.assertEquals(len(output), 9, "Output size was not equal to input size!")
+        self.assertEquals(map(itemgetter(0), output), sorted(map(itemgetter(0), output)), 
+                          "Output was not time sorted!")        
+        
     def testLexicalMerge(self):
         os.write(self.tempfiles[0][0], "\n".join(['1 one', '300 threehundred', '5 five', 
                                             '500 fivehundred']))
@@ -174,6 +214,32 @@ class MergeTestCase(unittest.TestCase):
         self.assertEquals(map(itemgetter(0), output), sorted(map(itemgetter(0), output)), 
                           "Output was not lexically sorted!")
         
+   
+class QPSTestCase(unittest.TestCase):
+    def setUp(self):
+        self.options = AttrDict({
+            "ignore": True,
+            "dt_re": r'^\[(.*?)\]',
+            "dateformat": "%d/%b/%Y:%H:%M:%S -0700",
+            "window_size": 15
+        })
+        self.fh = StringIO(
+            '[10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[10/Oct/2000:13:55:38 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[10/Oct/2000:13:56:59 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[10/Oct/2000:13:57:01 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[11/Oct/2000:14:01:00 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[11/Oct/2000:14:01:13 -0700] "GET /apache_pb.gif HTTP/1.0" \n' \
+            '[11/Oct/2000:14:01:14 -0700] "GET /apache_pb.gif HTTP/1.0" \n'
+        )
+    def testQps(self):
+        blocks=0
+        qs=[]
+        for q in qps(fh=self.fh, **self.options):
+            blocks+=1
+            qs.append(q)
+        self.assertEquals(blocks, 3, "qps output size different than expected: %s" % str(blocks))
+            
         
 class PlotTestCase(unittest.TestCase):
     def setUp(self):
